jest.mock('@angular/router');

import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, Router } from '@angular/router';
import { of } from 'rxjs';

import { IParametroAplicacion, ParametroAplicacion } from '../parametro-aplicacion.model';
import { ParametroAplicacionService } from '../service/parametro-aplicacion.service';

import { ParametroAplicacionRoutingResolveService } from './parametro-aplicacion-routing-resolve.service';

describe('Service Tests', () => {
  describe('ParametroAplicacion routing resolve service', () => {
    let mockRouter: Router;
    let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
    let routingResolveService: ParametroAplicacionRoutingResolveService;
    let service: ParametroAplicacionService;
    let resultParametroAplicacion: IParametroAplicacion | undefined;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        providers: [Router, ActivatedRouteSnapshot],
      });
      mockRouter = TestBed.inject(Router);
      mockActivatedRouteSnapshot = TestBed.inject(ActivatedRouteSnapshot);
      routingResolveService = TestBed.inject(ParametroAplicacionRoutingResolveService);
      service = TestBed.inject(ParametroAplicacionService);
      resultParametroAplicacion = undefined;
    });

    describe('resolve', () => {
      it('should return IParametroAplicacion returned by find', () => {
        // GIVEN
        service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultParametroAplicacion = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultParametroAplicacion).toEqual({ id: 123 });
      });

      it('should return new IParametroAplicacion if id is not provided', () => {
        // GIVEN
        service.find = jest.fn();
        mockActivatedRouteSnapshot.params = {};

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultParametroAplicacion = result;
        });

        // THEN
        expect(service.find).not.toBeCalled();
        expect(resultParametroAplicacion).toEqual(new ParametroAplicacion());
      });

      it('should route to 404 page if data not found in server', () => {
        // GIVEN
        jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as ParametroAplicacion })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultParametroAplicacion = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultParametroAplicacion).toEqual(undefined);
        expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
      });
    });
  });
});
